<?php

namespace Drupal\splitting_ui\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\splitting_ui\SplittingManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Displays splitting CSS selector target.
 *
 * @internal
 */
class SplittingAdmin extends FormBase {

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Splitting manager.
   *
   * @var \Drupal\splitting_ui\SplittingManagerInterface
   */
  protected $targetManager;

  /**
   * Constructs a new Splitting object.
   *
   * @param \Drupal\splitting_ui\SplittingManagerInterface $target_manager
   *   The Splitting target manager.
   * @param \Symfony\Component\HttpFoundation\Request $current_request
   *   The current request.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   */
  public function __construct(SplittingManagerInterface $target_manager, Request $current_request, FormBuilderInterface $form_builder) {
    $this->targetManager  = $target_manager;
    $this->currentRequest = $current_request;
    $this->formBuilder    = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('splitting.target_manager'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('form_builder'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'splitting_admin_form';
  }

  /**
   * {@inheritdoc}
   *
   * @param array $form
   *   A nested array form elements comprising the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string $splitting
   *   (optional) CSS Selector to be added to Splitting.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $splitting = '') {
    $keys = $this->currentRequest->query->get('search');

    /** @var \Drupal\splitting_ui\Form\SplittingFilter $form */
    $form['splitting_admin_filter_form'] = $this->formBuilder->getForm(SplittingFilter::class, $keys);

    $rows   = [];
    $header = [$this->t('Target'), $this->t('Operations')];
    $result = $this->targetManager->findAll($keys);
    foreach ($result as $target) {
      $row             = [];
      $row[]           = $target->target;
      $links           = [];
      $links['edit']   = [
        'title' => $this->t('Edit'),
        'url'   => Url::fromRoute('splitting.edit', ['splitting' => $target->sid]),
      ];
      $links['delete'] = [
        'title' => $this->t('Delete'),
        'url'   => Url::fromRoute('splitting.delete', ['splitting' => $target->sid]),
      ];
      $row[]           = [
        'data' => [
          '#type'  => 'operations',
          '#links' => $links,
        ],
      ];
      $rows[]          = $row;
    }

    $form['splitting_admin_table'] = [
      '#type'   => 'table',
      '#header' => $header,
      '#rows'   => $rows,
      '#empty'  => $this->t('No splitting CSS selector available. <a href=":link">Add target</a> .', [
        ':link' => Url::fromRoute('splitting.add')
          ->toString(),
      ]),
    ];

    $form['pager'] = ['#type' => 'pager'];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // @todo Add operations to splitting list
    /*$search = trim($form_state->getValue('filter'));
    $this->targetManager->findAll($search);
    $form_state->setRedirect('splitting.admin');*/
  }

}
